describe("books component test", function() {
  var component;
  var sandbox;
  var booksService;
  var updateBooksBindStub = function() {};
  var updateErrorBindStub = function() {};
  var observable = {subscribe: function() {} };
  var sortPipe = {transform: function(data) { return data; }};
  
  beforeEach(function() {
  	booksService = {
      get: function() {},
    };   

    component = new app.BooksComponent(booksService, sortPipe);

    sandbox = sinon.sandbox.create();    

    sandbox.stub(component.updateBooks, 'bind')
           .withArgs(component)
           .returns(updateBooksBindStub);

    sandbox.stub(component.updateError, 'bind')
           .withArgs(component)
           .returns(updateErrorBindStub);

    sandbox.stub(booksService, 'get')
           .withArgs()
           .returns(observable);
  });                                
  
  afterEach(function() {
    sandbox.restore();
  });
  
  it('should pass the canary test', function() {
    expect(true).to.be.true;
  });   
  
  it('should set proper selector', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.BooksComponent)[0];
      
    expect(annotations.selector).to.be.eql('book-list');
  });
  
  it('should set templateURL to books.html', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.BooksComponent)[0];
      
    expect(annotations.templateUrl).to.be.eql('books.html');
  });
  
  it('books should be empty on create', function() {
    expect(component.books).to.be.eql([]);
  });                                     
  
  it('message should be empty on create', function() {
    expect(component.message).to.be.eql('');
  });                                       
  
  it('getBooks should call service and register handlers', function() {
    var observableMock = sandbox.mock(observable)
                                .expects('subscribe')
                                .withArgs(
                                  updateBooksBindStub, updateErrorBindStub);

    component.getBooks();
    observableMock.verify();
  });                       
  
  it('updateBooks should update books', function() {
    var booksStub = [{name: 'sample'}];
    
    component.updateBooks(booksStub);
    
    expect(component.books).to.be.eql(booksStub);
  });

  it('updateError should update message', function() {
    var error = 'some message';
    component.updateError(error);
    expect(component.message).to.be.eql(error);
  });                                          
  
  it('updateBooks should call transform', function() {
    var fakeInput = '...some fake input...';
    var fakeOutput = '...some fake output...';
    
    sandbox.stub(sortPipe, 'transform')
           .withArgs(fakeInput)
           .returns(fakeOutput);
    
    component.updateBooks(fakeInput);
    
    expect(component.books).to.be.eql(fakeOutput);
  });

  it('should have proper providers', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.BooksComponent)[0];
      
    var providers = annotations.providers;
      
    expect(providers).to.be.eql([
        ng.http.HTTP_PROVIDERS,
        app.BooksService,
        app.BooksSortPipe
      ]);
  });
  
  it('should inject dependencies', function() {
    var parameters = Reflect.getMetadata('parameters', app.BooksComponent);
                     
    expect(parameters[0]).to.be.eql([app.BooksService]);
    expect(parameters[1]).to.be.eql([app.BooksSortPipe]);
  });
  

  it('ngOnInit should call getBooks', function() {
    var getBooksMock = sandbox.mock(component)
                              .expects('getBooks');

    component.ngOnInit();
    
    getBooksMock.verify();
  });
});
