describe('books service tests', function() {
  var sandbox;
  var service;
  var observable;
  var http;
  
  beforeEach(function() {
    sandbox = sinon.sandbox.create();
    http = {
      get: function() {}
    };
    
    service = new app.BooksService(http);

    observable = {
      map: function() {},
      catch: function() {}
    };
    
    sandbox.stub(http, 'get')
         .withArgs('/books')
         .returns(observable);

    sandbox.stub(observable, 'map')
         .withArgs(service.extractData)
         .returns(observable);

    sandbox.stub(observable, 'catch')
         .withArgs(service.returnError)
         .returns(observable);
  }); 
  
  afterEach(function() {
    sandbox.restore();
  });
  
  it('get should call http.get and register handlers', function() {
    expect(service.get()).to.be.eql(observable);
    expect(http.get.calledWith('/books')).to.be.true;
    expect(observable.map.calledWith(service.extractData)).to.be.true;
    expect(observable.catch.calledWith(service.returnError)).to.be.true;
  });
  
  it('extractData should return json() response', function() {           
    var fakeData = '...stuff...';
    var fakeResponse = {status: 200, json: function() { return fakeData; }}
    
    expect(service.extractData(fakeResponse)).to.be.eql(fakeData);
  });

  it('extractData should throw exception of status not 200', function() {           
    var fakeResponse = {status: 404}

    var call = function() { service.extractData(fakeResponse); }
    
    expect(call).to.throw('Error, status: 404')
  });                                          
  
  it('returnError should call Observable.throw', function() {
    var error = {message: 'oops'};
    
    var observableMock = 
      sandbox.mock(Rx.Observable)
           .expects('throw')
           .withArgs('oops');
    
    service.returnError(error);
    observableMock.verify();
  });

  
  it('returnError should call Observable.throw with no error.message', function() {
    var error = {};
    
    var observableMock = 
      sandbox.mock(Rx.Observable)
           .expects('throw')
           .withArgs('Error: request failed');
    
    service.returnError(error);
    observableMock.verify();
  });
  
  it('should inject dependency', function() {
    var parameters = Reflect.getMetadata('parameters', app.BooksService);
    
    expect(parameters[0]).to.be.eql([ng.http.Http]);
  });
});